#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import time
import datetime
from time import strftime,localtime

class timer:
  def __init__(self):
    self.recordings = []
    self.mark = None

  def hms(self):
    return self.now().strftime('%H:%M:%S')

  def now(self):
    return datetime.datetime.now()

  def delta_t(self, time):
    return (self.now() - time).total_seconds()

  def start(self):
    print("Starting: {}".format(self.hms()))
    self.mark = self.now()

  def stop(self):
    print("Stopping: {}".format(self.hms()))
    self.recordings.append(self.delta_t(self.mark))
    self.mark = None

  def filetime():
    return strftime("%Y_%m_%d_%H_%M_%S", localtime())


# In[ ]:


t = timer()


# In[ ]:


t.start()


# In[ ]:


t.stop()
print(t.recordings)


# In[ ]:


timer.filetime()


# In[ ]:







#!/usr/bin/env python
# coding: utf-8

# In[8]:


import ipywidgets as widgets
import IPython
import copy

class question:
  def __init__(self, string, style, options, answer =None):
    self.text = string
    self.style = style
    self.options = options
    self.answer = answer


# In[9]:


class attributeAnalyzer:
  def __init__(self, questions=[]):
    self.obj = None
    q_holder = []
    for q in questions:
      q_holder.append(copy.copy(q))
    self.questions = q_holder

  def new_question(self, *args, **kwargs):
    self.questions.append(question(*args, **kwargs))

  def has_blank_answers(self):
    for q in self.questions:
      if q.answer is None:
        return True
    return False

  def collect_input(self):
    for q in self.questions:
      q.selector = getattr(self, q.style)(q)

  def keyboardinput(self, q):
    q.answer = input(q.text)

  def __str__(self):
    out = ""
    for q in self.questions:
      out += ("{}: {}\n".format(q.text,q.answer))
    return out


# In[10]:


aa = attributeAnalyzer()
aa.new_question("how are you?", style="keyboardinput", options = {"this good": 1, "this bad": -1})
aa.new_question("What is 1+1", style="keyboardinput", options={"first":-4, "second":3, "third": 2})
assert aa.has_blank_answers()


# In[11]:


for q in aa.questions:
  q.answer = "something"

assert aa.has_blank_answers() is False


# In[13]:


b = attributeAnalyzer(aa.questions)

assert b.has_blank_answers() == False
assert len(b.questions) == 2


# In[ ]:







#!/usr/bin/env python
# coding: utf-8

# In[ ]:


from google.colab import drive
drive.mount('/content/drive')


# In[ ]:


import os
os.chdir("/content/drive/My Drive/repos/MusicalPy") 


# In[ ]:





# In[ ]:


import json
import os
class fileHelper():
  def __init__(self):
    self.file_extension = "file_handler"
    self.project_root = "file_handler"
    self.last_save_location

  def cls(self):
    b = self.__init__()
    b.file_extension = "NONE"
    return b

  def default_filename(self):
    return "file_handler"

  def touch_directory(filename):
    if(len(filename.split("/")) > 0):
      path = "{}".format("/".join(filename.split("/")[:-1]))
      os.makedirs(path,exist_ok=True)

  def save(self, filename = None):
    if filename is None:
      filename = self.default_filename()
    filename = "{}/{}.{}".format(self.project_root, filename, self.file_extension)
    fileHelper.touch_directory(filename)

    with open(filename, 'w') as outfile:
      json.dump(self.toJson(), outfile)

    self.last_save_location = filename
    return filename


  def toJson(self):
    return json.dumps(self, default=lambda o: o.__dict__)

  def load_json(filename):
    with open( filename, "rb" ) as infile:
      return json.loads(json.load(infile))

  def load(filename, fresh_class):
    json = fileHelper.load_json(filename)
    return fileHelper.convert(json, fresh_class)

  def convert(json, dclass):
    for x in json:
      dclass.__dict__[x] = json[x]
    return dclass


# In[ ]:


class tester(fileHelper):
  def __init__(self): 
    self.file_extension = "filehelpertester"
    self.project_root = "testing"
    self.data = "fuubar"


# In[ ]:


class adder(fileHelper):
  def __init__(self, *args):
    self.file_extension = "assert_test"
    self.project_root = "assert"
    self.obs = args

  def add(self):
    total = 0
    for o in self.obs:
      total += o
    return total


# In[ ]:


a = adder(1, 3, 4)
assert a.add() == 8
a.save(filename = "foobar")
b = adder.load("assert/foobar.assert_test", adder())
assert b.add() == 8


# In[ ]:







#!/usr/bin/env python
# coding: utf-8

# In[1]:


from google.colab import drive
drive.mount('/content/drive')


# In[2]:


import os
os.chdir("/content/drive/My Drive/repos/MusicalPy") 
from midimusic.api_interaction.completion import *
from midimusic.api_interaction.fetcher import *
from midimusic.helpers.beeper import *


# In[3]:


import json
from time import localtime, strftime

class generator(fileHelper):
  def __init__(self, seed = None, verbose = True, play_tones = True):
    self.payload = seed
    self.result = None
    self.verbose = verbose
    self.play_tones = play_tones
    self.file_extension = "generator"
    if seed is not None:
      self.project_root = seed.project_root
    else:
      self.project_root = None

  def fetch(self):
    return self.fetch_until_response(max_fetches = 1)

  def fetch_until_response(self, max_fetches = 5):
    if self.payload is None:
      raise Exception("Payload is None")
    if self.project_root is None:
      raise Exception("no project root")
    count = 0
    while(not self.payload.has_children() and count < max_fetches):
      count += 1
      self.payload.fetch_count = count
      self.log("On fetch {}/{}".format(count, max_fetches))
      fetcher.fetch(self.payload)

    if(self.payload.has_children()):
      if self.play_tones:
        beep_beep("complete") # beep beep i'm a jeep
      self.log("successful fetching".format(count, max_fetches))
      self.result = self.payload
      self.payload = None
    else:
      self.log("failed")
      if self.play_tones:
        beep_beep("error")
      self.result = None

  def keep_fetching(self, max_fetches=3, loops=5):
    count = 0
    while(count < loops):
      self.log("On loop {}/{}".format(count+1, loops))
      self.fetch_until_response(max_fetches)
      if(self.payload is None): #successful fetching
        count += 1
        self.select_next_payload()
      else:
        break #failed a fetch
    return count


  def select_next_payload(self):
    #probably best to overwrite this method with something smarter choosing the children
    self.payload = self.result.children[-1]

  def default_filename(self):
    if self.payload is not None:
      return self.payload.default_filename()
    if self.result is not None:
      return self.result.default_filename()
    return f"generators/failed/#{timer.filetime()}"

  def log(self, msg):
    if self.verbose:
      print(msg)

  def load_objects(filename):
    json = generator.load_json(filename)
    return generator.objectize(json)

  def objectize(json):
    if json is None:
      return json
    else:
      partial = fileHelper.convert(json, generator(None, True, True))
      partial.payload = completion.objectize(partial.payload)
      partial.result  = completion.objectize(partial.result)
      return partial



# In[7]:


load = generator.load_objects("testing/gen save load/thebeatles/0.8/piano/2020_08_08_21_54_50.generator")


# In[8]:


assert isinstance(load.result, completion)


# In[8]:







#!/usr/bin/env python
# coding: utf-8

# In[ ]:


from google.colab import drive
drive.mount('/content/drive')


# In[ ]:


import os
os.chdir("/content/drive/My Drive/repos/MusicalPy") 


# In[ ]:


get_ipython().system(u'pip install mir_eval')
get_ipython().system(u'pip install librosa')
get_ipython().system(u'pip install pretty_midi')


# In[ ]:


from midimusic.analyzers.attributeAnalyzer import *
from midimusic.api_interaction.musenetSettings import *
from midimusic.api_interaction.spider import spider
from midimusic.helpers.timer import *
from midimusic.helpers.fileHelper import *


# In[ ]:


import mir_eval.display
import librosa.display
import matplotlib.pyplot as plt
import pretty_midi
from os import path
import pandas as pd
import IPython
import copy

class midiAnalyzer(attributeAnalyzer):
  def __init__(self, spi=None):
    if(spi is not None):
      self.df = self.initialize_from_spider(spi)
      
  def png_loc_from_midi_loc(midi_location):
    return "{}.png".format(".".join(midi_location.split(".")[:-1]))

  def plot_piano_roll(pm, start_pitch, end_pitch, fs=100):
      librosa.display.specshow(pm.get_piano_roll(fs)[start_pitch:end_pitch],
                              hop_length=1, sr=fs, x_axis=None, y_axis=None,
                              fmin=pretty_midi.note_number_to_hz(start_pitch))
      
  def save_piano_roll_image(midi_location, start_pitch, end_pitch, fs, width, height, overwrite):
    if midi_location is None or len(midi_location) < 3:
      return ""
    png_out = midiAnalyzer.png_loc_from_midi_loc(midi_location)
    if not overwrite and path.exists(png_out):
      return png_out
    
    print(f"loading {midi_location}")
    plt.figure(figsize=(width, height))
    pm = pretty_midi.PrettyMIDI(midi_location)
    midiAnalyzer.plot_piano_roll(pm, start_pitch, end_pitch)
    fileHelper.touch_directory(png_out)
    plt.savefig(png_out,bbox_inches='tight')
    return png_out

  def generate_images_from_loaded(self, start_pitch = 20, end_pitch = 90, overwrite = False, fs=100, width=8, height=8):
    self.df["png_location"] = [midiAnalyzer.save_piano_roll_image(x, start_pitch, end_pitch, fs, width, height, overwrite) for x in self.df["midi_location"]]
      
  def ask(self, id, question, overwrite=False):
    if not pd.isnull(self.df["png_location"].values[id]) and len(self.df["png_location"].values[id]) > 5:
      IPython.display.clear_output(wait=True)
      IPython.display.display(IPython.display.Audio(pretty_midi.PrettyMIDI(self.df["midi_location"].values[id]).synthesize(fs=16000), rate=16000, autoplay=True))
      IPython.display.display(IPython.display.Image(self.df["png_location"].values[id]))
      print(question)
      self.df[question].values[id] = input()

  def new_question(self, text):
    try:
      self.df[text]
      return False
    except:
      self.df[text] = np.NaN
      return True

  def ask_unanswered(self, question, save_every=10):
    for i in range(len(self.df)):
      if pd.isnull(self.df[question].values[i]):
        self.ask(i, question)
        if i%save_every == 0:
          self.save()
      else:
        print(f"Already answered index {i}")

  def save(self, filename = None):
    if filename is None:
      filename = f"analysis/{timer.filetime()}"
    outfile = f"{self.project_root}/{filename}"
    fileHelper.touch_directory(outfile)
    data = {}
    for key in self.__dict__.keys():
      if isinstance( self.__dict__[key], pd.DataFrame):
        data[key] = self.__dict__[key].to_json()
      else:
        data[key] = self.__dict__[key]

    with open(outfile, 'w') as out:
      json.dump((data), out)
    return outfile

  def load_from_save(infile):
    with open(infile, 'r') as json_file:
      data = json.load(json_file)
    m = midiAnalyzer()
    m.df = pd.DataFrame(json.loads(data["df"]))
    m.project_root = data["project_root"]
    return m

  def initialize_from_spider(self, s):
    self.project_root = s.project_root
    out = []
    for item in s.logger.data:
      tmp = {
            "composer":        item.settings.genre,
            "temp":            item.settings.temp,
            "instrumentation": "_".join([x for x in item.settings.instrumentation if item.settings.instrumentation[x]]),
            "max_fetches":     s.max_fetches,
            "loops":           s.loops, 
            "count":           item.loop_count,
            "midi_location":   item.midi_location,
            "analysis":        ""
            }
      if item.midi_location:
        png = midiAnalyzer.png_loc_from_midi_loc(item.midi_location)

        if path.exists(png):
          tmp["png_location"] = png
        else:
          tmp["png_location"] = ""
      out.append(tmp)

    return pd.DataFrame(out)




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import IPython, pretty_midi

def beep_beep(action):
  sound = pretty_midi.Instrument(program=42)
  if action == "complete":
    for note_name in ['C5', 'E5', 'G5']:
      note_number = pretty_midi.note_name_to_number(note_name)
      note1 = pretty_midi.Note(velocity=100, pitch=note_number, start=0, end=.5)
      note2 = pretty_midi.Note(velocity=100, pitch=note_number, start=.25, end=.75)
      sound.notes.append(note1)
      sound.notes.append(note2)
  if action == "error":
    for note_name in ['C3', 'E3', 'G3']:
      note_number = pretty_midi.note_name_to_number(note_name)
      note1 = pretty_midi.Note(velocity=100, pitch=note_number, start=0, end=.25)
      note2 = pretty_midi.Note(velocity=100, pitch=note_number, start=.75, end=1)
      sound.notes.append(note1)
      sound.notes.append(note2)
  IPython.display.display(IPython.display.Audio(sound.synthesize(fs=16000), rate=16000, autoplay=True))


# In[ ]:





# In[ ]:







#!/usr/bin/env python
# coding: utf-8

# In[ ]:





# In[3]:


from google.colab import drive
drive.mount('/content/drive')


# In[4]:


import os
os.chdir("/content/drive/My Drive/repos/MusicalPy") 


# In[5]:


from midimusic.helpers.fileHelper import *
from midimusic.api_interaction.musenetSettings import *
from midimusic.api_interaction.completion import *
from midimusic.api_interaction.generator import *
#import magicalMusic.core


# In[23]:


class spider(fileHelper):
  def __init__(self):
    self.file_extension = "spider"
    self.temps = [.7, .9]
    self.composers = musenetSettings.legal_composers()
    self.instrs = [["piano"], ["guitar"], ["piano", "strings"]]
    self.max_fetches = 2
    self.loops = 2
    self.seeds = 3
    self.project_root = None
    self.trial_complete_tones = False
    self.play_final_song = True
    self.verbose = True
    self.logger = logger()

  def default_filename(self):
    return f"spiders/{timer.filetime()}"

  def pre_check(self):
    print(f"Temps: {self.temps}")
    print(f"Commposers: {self.composers}")
    print(f"instruments: {self.instrs}")
    print(f"Max Fetches: {self.max_fetches}")
    print(f"Loops: {self.loops}")
    print(f"Project: {self.project_root}")
    print(f"Number of seeds: {self.seeds}")
    print(f"Data: {len(self.logger.data)}, Queried: {len(self.logger.queried)}")
    print(f"Trial tones: {self.trial_complete_tones} Play Songs: {self.play_final_song}")

  def set_project(self):
    self.project_root = f"loops {self.loops}/max fetches {self.max_fetches}/"

  def info_arr(self, c, t, i):
    return [c, t, i]

  def fetch_from_musenet(self):
    if self.project_root is None:
      raise Exception("Need to set a project_root")
    self.logger.project_root = self.project_root

    count = 1
    total_ittrs = len(self.composers)*len(self.temps)*len(self.instrs)*self.seeds
    for c in self.composers:
      for t in self.temps:
        for i in self.instrs:
          info = {"composer": c,
                  "instrumentation":i,
                  "temp":t}
          self.logger.log("[{}/{}] fetching seed: {}".format(count, total_ittrs, info), self.verbose)
          c_t_i = self.info_arr(c = c, t = t, i = i)
          if(c_t_i in self.logger.queried):
            count += self.seeds
            print("previously queried")
            continue
          info["children"] = {}
          for s in range(self.seeds):
            count += 1
            a = completion(genre = c, instrumentation=i, temp=t, project_root = self.project_root)
            g = generator(a)
            assert g.project_root == self.project_root
            g.play_tones = self.trial_complete_tones
            self.logger.log("child {}/{}".format(s+1, self.seeds), self.verbose)
            child_info = {}
            loops = g.keep_fetching(max_fetches=self.max_fetches, loops = self.loops)
            if loops == 0:
              final_result = a
              self.logger.log("Loops = 0", self.verbose)
            else:
              fetches = -1
              if g.result is not None:
                fetches = g.result.fetch_count
                g.select_next_payload()
              final_result = g.payload
              final_result.fetch_count = fetches
              self.logger.log(final_result.save_midi(), self.verbose)
            final_result.loop_count = loops
            final_result.save()
            info["children"][s] = final_result
            if loops > 0:
              final_result.play(play_each = self.play_final_song)
          self.logger.queried.append(c_t_i)
          self.logger.data.append(info)

          self.logger.log(f"Backing up data for spider {self.save()}", self.verbose)
          
      
  def load_objects(filename):
    json = spider.load_json(filename)
    return spider.objectize(json)

  def objectize(json):
    if json is None:
      return json
    else:
      partial = fileHelper.convert(json, spider())      
      partial.logger = logger.objectize(partial.logger)
      return partial



# In[7]:


class logger(fileHelper):
  def __init__(self): 
    self.file_extension = "logs"
    self.queried = []
    self.data = []

  def objectize(json):
    partial = fileHelper.convert(json, logger())
    data = []
    for node in partial.data:
      if "totalTime" not in node.keys(): #legacy spider
        for child in node["children"]:
          data.append(completion.objectize(node["children"][child]))
      else:
        data.append(completion.objectize(node))

    partial.data= data
    return partial

  def default_filename(self):
    return f"logs/logs_{timer.filetime()}"

  def log(self, msg, bool):
    if bool:
      print(msg)


# In[9]:


s = spider.load_objects("(arch) loops 4/max fetches 1/spiders/2020_08_09_06_17_50.spider")


# In[10]:


assert s.logger.data[3].settings.genre == "chopin"


# In[11]:


class fixed_filename(spider):
  def __init__(self):
    super().__init__()
    self.fixed_time = timer.filetime()

  def default_filename(self):
    return f"spiders/initiated_{self.fixed_time}"
  
  def load_objects(filename):
    json = fixed_filename.load_json(filename)
    return fixed_filename.objectize(json)

  def objectize(json):
    if json is None:
      return json
    else:
      partial = fileHelper.convert(json, fixed_filename()) 
      partial.logger = logger.objectize(partial.logger)
      
      return partial
  



# In[12]:


f = fixed_filename.load_objects("(arch) loops 4/max fetches 1/spiders/initiated_2020_08_09_06_28_32.spider")


# In[13]:


assert len(f.logger.data) == 324


# In[16]:


c = completion(project_root="testing")


# In[17]:


g = generator(c)


# In[ ]:







#!/usr/bin/env python
# coding: utf-8

# In[1]:


class musenetSettings:
  def __init__(self, enc="", audioFormat = "mp3", genre="thebeatles", truncation = 27, temp=.8, length=225, instrumentation = ["piano"]):
    self.project_root = "musenet_settings"
    self.enc = enc
    self.genre = genre
    self.temp = temp
    self.legal_instruments = "piano strings winds drums harp guitar bass".split(" ")
    self.turn_on(instrumentation)
    self.length = length
    self.truncation = truncation
    self.audioFormat = audioFormat
   
  def legal_composers():
    return ['chopin',
            'mozart',
            'rachmaninoff',
            'country',
            'bach',
            'beethoven',
            'thebeatles',
            'franksinatra',
            'tchaikovsky'] 

  def turn_on(self, instruments):
    self.toggle_instruments(instruments, True)

  def turn_off(self, instruments):
    self.toggle_instruments(instruments, False)

  def toggle_instruments(self, instruments, state):
    instr_dict = {}
    lower_instruments = [x.lower() for x in instruments]
    for item in self.legal_instruments:
      if item in lower_instruments:
        instr_dict[item] = state
      else:
        instr_dict[item] = not state
    self.instrumentation = instr_dict

  def instrumentation_settings(self, piano=False, strings=False,winds=False,drums=False,harp=False,guitar=False,bass=False):
    instr = {"piano":piano,"strings":strings,"winds":winds,
             "drums":drums,"harp":harp,"guitar":guitar,"bass":bass}
    return instr

  def __str__(self):
    s = """genre: {}\ntemp: {}\ninstrumentation: {}\ntokens: {}\ntruncation: {}\naudio format: {}\nenc: <{}>"""
    return s.format(self.genre, self.temp, self.instrumentation, self.length, self.truncation, self.audioFormat, self.enc)


# In[2]:


assert musenetSettings(instrumentation = ["piano", "winds"]).instrumentation == {'bass': False,
 'drums': False,
 'guitar': False,
 'harp': False,
 'piano': True,
 'strings': False,
 'winds': True}


# In[3]:


assert musenetSettings.legal_composers()[0] == "chopin"


# In[ ]:







#!/usr/bin/env python
# coding: utf-8

# In[1]:


import requests
import json
import copy

class fetcher:
  def fetch(c):
    response = fetcher.fetch_from_musenet(c)
    if fetcher.erred(response):
      return fetcher.problems(response)

    c.add_children(fetcher.process_raw_musenet(response))
    return c

  def fetch_from_musenet(c):
    settings = c.settings
    p = {"genre":settings.genre,
       "instrument":settings.instrumentation,
       "encoding":settings.enc,
       "temperature":settings.temp,
       "truncation":settings.truncation,
       "generationLength": settings.length,
       "audioFormat":settings.audioFormat}

    h = {
      "Content-Type": "application/json",
      "path": "/sample"
    }

    r = requests.post("https://musenet.openai.com/sample",
        data = json.dumps(p),
        headers = h
    )
    return r.content.decode()

  def process_raw_musenet(response):
    return json.loads(response)["completions"]

  def problems(response):
    try:
      out = fetcher.process_raw_musenet(response)
      return None
    except Exception as e:
      return e

  def erred(response):
    return isinstance(fetcher.problems(response), Exception)


# In[ ]:







#!/usr/bin/env python
# coding: utf-8

# In[ ]:


from google.colab import drive
drive.mount('/content/drive')


# In[ ]:


import os
os.chdir("/content/drive/My Drive/repos/MusicalPy") 


# In[ ]:


from midimusic.helpers.fileHelper import *
from midimusic.helpers.timer import *
from midimusic.api_interaction.musenetSettings import *

import copy
import time
import IPython
import pretty_midi
import numpy as np
from time import gmtime, strftime
import os


# In[ ]:


class pictureManipulation:
  def plot_piano_roll(pm, start_pitch, end_pitch, fs=100):
      librosa.display.specshow(pm.get_piano_roll(fs)[start_pitch:end_pitch],
                              hop_length=1, sr=fs, x_axis=None, y_axis=None,
                              fmin=pretty_midi.note_number_to_hz(start_pitch))
      
  def png_location(midi_location):
    return "{}.png".format(".".join(midi_location.split(".")[:-1]))

  def save_piano_roll_image_from_midi_location(midi_location, start_pitch, end_pitch, fs=100, width=8, height=8):
    pm = pretty_midi.PrettyMIDI(midi_location)
    png_out = png_location(midi_location)
    if(path.exists(png_out)):
      c.png_location = png_out
      print("image exists: {}".format(png_out))
      return True
    plt.figure(figsize=(width, height))
    pictureManipulation.plot_piano_roll(pm.to_midi(), start_pitch, end_pitch)
    fileHandler.touch_directory(png_out)
    plt.savefig(png_out,bbox_inches='tight')
    return png_out


# In[ ]:


class audioManipulation:
  def __init__(self, tracks):
    self.tracks = tracks

  def instrument_name_to_program(self, string):
    lookup = {
      "piano": "Acoustic Grand Piano",
      "bass": "Acoustic Bass",
      "winds": "Recorder",
      "drums": "Synth Drum",
      "harp": "Orchestral Harp",
      "guitar": "Acoustic Guitar (nylon)",
      "strings": "Violin"
    }
    try:
      return pretty_midi.instrument_name_to_program(lookup[string].title())
    except:
      return pretty_midi.instrument_name_to_program(string.title())
        
  def append_notes_to_inst(self, inst, notes, velocity=100):
    for note in notes:
        inst.notes.append(pretty_midi.Note(velocity, note["pitch"], note["time_on"], note["time_on"] + note["duration"]))
    return inst

  def to_midi(self):
    pm = pretty_midi.PrettyMIDI(initial_tempo=120)
    if self.tracks is not None:
      for track in self.tracks:
        inst = pretty_midi.Instrument(program=self.instrument_name_to_program(track['instrument']), is_drum=False)
        inst = self.append_notes_to_inst(inst, track['notes'])
        pm.instruments.append(inst)
    return pm

  def play(self, play_each = True, sleep=False):
    IPython.display.display(IPython.display.Audio(self.to_midi().synthesize(fs=16000), rate=16000, autoplay=play_each))
    if sleep:
        time.sleep(self.totalTime + .25)

  def play_gens(self, **kwargs):
    for c in self.children:
      c.play(**kwargs)

  def save_midi(self, filename = None):
    if filename is None:
      filename = "{}/{}.mid".format(self.project_root, self.default_filename())
    fileHelper.touch_directory(filename)
    self.to_midi().write(filename)
    self.midi_location = filename
    return filename


# In[ ]:


from midimusic.helpers.fileHelper import *

class completion(fileHelper, audioManipulation, pictureManipulation):
  def __init__(self, project_root = None, file_extension = "completion", **kwargs):
    self.settings = musenetSettings(**kwargs)
    self.children = []
    self.totalTime = 0
    self.tracks = None
    self.midi_location = None
    self.png_location = None
    self.file_extension = file_extension
    self.loop_count = None
    self.fetch_count = None
    if project_root is not None:
      self.project_root = project_root
    #else all is blank

  def add_children(self, processed_musenet):
    for item in processed_musenet:
      tmp = completion()
      tmp.settings = copy.copy(self.settings)
      tmp.settings.enc = ("{} {}".format(self.settings.enc.strip(), item['encoding'].strip())).strip()
      tmp.totalTime = item["totalTime"]
      tmp.tracks = item["tracks"]
      tmp.project_root = self.project_root
      self.children.append(tmp)

  def has_children(self):
    return len(self.children) > 0
  
  def default_filename(self):
    instr = self.settings.instrumentation
    return "{}/{}/{}/{}".format(self.settings.genre, 
                                self.settings.temp, "_".join([x for x in instr.keys() if instr[x] is True]), 
                                timer.filetime())

  def load_objects(filename):
    json = completion.load_json(filename)
    return completion.objectize(json)

  def objectize(json):
    if json is None:
      return None
    partial = fileHelper.convert(json, completion())
    partial.settings = fileHelper.convert(partial.settings, musenetSettings())
    if not partial.has_children():
      return partial
    else:
      for i in range(len(partial.children)):
        partial.children[i] = completion.objectize(partial.children[i])
      return partial


# In[ ]:


c = completion()
b = completion("testing/save-load")


# In[ ]:


c.settings


# In[ ]:


b.children.append(c)
b.children.append(c)


# In[ ]:


d = completion()
c.children.append(d)


# In[ ]:


out = b.save()


# In[ ]:


d = completion.load_objects(out)


# In[ ]:


assert isinstance(d.children[0].children[0].settings, musenetSettings)


# In[ ]:




